<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz data class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class BookzoArrangement extends BookzoResponseData {
	
	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	protected $images;
	
	/**
	 * An alias for this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $alias;
	
	/**
	 * The seo url to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $link;
	
	/**
	 * The seo url to this object
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $objects;
		
	/**
	 * Bind data to object.
	 * Override for parent class so we can set our own data
	 *
	 * @param   object  $item  The data object
	 * @return  void
	 * @access	proteced
	 * @since   1.0.0
	 */
	protected function bind($item)
	{		
		// set general properties
		$this->setProperties((array) $item);
		
		// set objects
		$objects = array();
		if ( is_array($item->objects) ) {
			foreach ( $item->objects as $v ) {
				$objects[] = $v->objectid;
			}
		}
		
		// set alias
		$this->set('objects', $objects);
		$this->set('alias', strtolower(str_replace(' ', '-', trim($this->translate('name')))));
	}
	
	/**
	 * Thumbnail return the first image found
	 *
	 * @param   none
	 * @return  string	the image html
	 * @access	public
	 * @since   1.0.0
	 */
	public function image( $type = 'medium', $fulltag = false )
	{
		$return = '';
		$type 	= $type == 'large' || $type == 'small' ? $type : 'medium';
		
		if ( $this->hasImages() ) 
		{
			$type 	= 'image'.strtolower($type);
			$src 	= $this->images[0];
			
			if ( isset($src->$type) ) {
				$return = $src->$type;
			}
		}
		
		if ( $return && $fulltag ) {
			return '<img src="'.$return.'" />';
		}
		
		return $return;
	}
	
	/**
	 * See if object has images
	 *
	 * @param   none
	 * @return  boolean		true if images found, false otherwise
	 * @access	public
	 * @since   1.0.0
	 */
	public function hasImages()
	{
		return count($this->images);
	}	
	
	/**
	 * Get object link
	 * We only build link if required 
	 *
	 * @param   none
	 * @return  link	string	the link for the object
	 * @access	proteced
	 * @since   1.0.0
	 */	
	public function getLink( $route = true, $addbase = false, $xhtml = true, $add = '' )
	{
		// build base link
		if ( is_null( $this->link ) ) 
		{
			$id	  		= $this->get('id');
			$alias 		= $this->get('alias');
			
			$this->set('link', 'layout=arrangement&aid='.$id.':'.$alias);
		}
		
		$link = $this->link;
		
		// add parts if needed 
		if ( $add ) {
			$link = $this->link . '&' . $add;
		}
		
		// route
		if ( $route ) {
			return $this->app->route->route( $link, $xhtml, $addbase );
		}
		
		return $this->link;
	}
	
	/**
	 * Get the object introtext
	 * If not set and altdescription is set we use the default description
	 *
	 * @param   none
	 * @return  $return	string	the text for the introtext
	 * @access	proteced
	 * @since   1.0.0
	 * @modded 	1.0.1 - added suffix support
	 */	
	public function getShortDescription( $length = 0 )
	{
		$return 	= $this->translate('shortdescription');
				
		// no introtext then use default decription
		if ( !strlen($return) && $this->app->config->get('template.arrangements_altdescription', 1) ) 
		{
			$length				= $length ? $length : (int) $this->app->config->get('template.arrangements_altdescriptionlength', 106);		
			$description 		= $this->translate('description');
			$return 			= substr($description, 0, $length);
			$suffix 			= $this->app->config->get('template.arrangements_altdescriptionsuffix', '...');
			
			if ( strlen($return) < strlen($description) && strlen(trim($suffix))  ) {
				$return .= ' ' . $suffix;
			}
		}
		
		return $return;
	}
	
	public function widget($name, $params = array(), $render = true)
	{
		// create param object
		$params = $this->app->registry->create($params);
		$key	= $name . $params->clean()->createkey();
		
		if ( !isset($this->widgets[$key]) ) 
		{
			switch ( $name ) 
			{
				// gallery widget
				case 'gallery' :
				
					if ( $widget = $this->app->widget->get('gallery') ) 
					{						
						foreach ( $this->images as $image ) {
							
							$imagetype = $params->get('images', 'large') == 'medium' ? 'imagemedium' : 'imagelarge';
							$lightbox  = $params->get('lightbox', 0) ? $image->imagelarge : null;
							
							$widget->addImage( $image->{$imagetype}, $image->imagesmall, (isset($image->name) ? $image->name : ''), $lightbox);
						}
					}
					
					break;
					
				// default
				default : 
				
					return '';				
			}
			
			// add parameters
			if ( $widget ) {
				$widget->setParams($params);
				$widget->setParam('id', $this->get('id'));
			}
			
			// add to internal cache
			$this->widgets[$key] = $widget;
		}
		
		// return only content if render is true
		if ( $render && $this->widgets[$key] !== false ) {
			return $this->widgets[$key]->render();
		}
			
		// return complete widget
		return $this->widgets[$key];
	}
	
	public function hasObject($id)
	{
		return in_array($id, $this->objects);
	}
	
}
