<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  Object
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Joomla Platform Object Class
 *
 * This class allows for simple but smart objects with get and set methods
 * and an internal error handler.
 *
 * @since       11.1
 * @deprecated  4.0
 */
class BookzoCalendar
{
	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	public $settings;
	
	public $blocks 	= array();
	
	public $prices 	= array();
	
	public $types 	= array();
	
	public $arrangements = array();
	
	public $mindate;
	
	public $maxdate;
	
	protected $activeperiod = false;
	
	
	
	/**
	 * Class constructor, overridden in descendant classes.
	 *
	 * @param   mixed  $properties  Either and associative array or another
	 *                              object to set the initial properties of the object.
	 *
	 * @since   11.1
	 */
	public function __construct($properties = array())
	{
		$this->app 			= Bookzo::getInstance();
		$this->settings		= new BRegistry;
		$this->search		= new BRegistry;
	}
		
	/**
	 * Add a block or a priceelement to the calendar
	 *
	 * @access		public
	 * @param		mixed	$block 	an object or array containing a price element or a period block
	 * @return 		self	for chaining
	 * @since		1.3.2
	 */
	public function addBlock($block)
	{
		// turn price into registry object
		$block  = new BRegistry($block);
		$block->set('arrival', 		$this->app->date->create( $block->get('arrival') )->format('Y-m-d')); 
		$block->set('departure', 	$this->app->date->create( $block->get('departure') )->format('Y-m-d')); 
				
		if ( $block->get('arrival') <= $block->get('departure') ) {
			if ( $block->get('price') ) {
				$this->prices[$block->get('arrival').'-'.$block->get('departure')] = $block;	
				ksort($this->prices);
				
				if ( !array_key_exists( (int) $block->get('nights'), $this->types) ) {
					$this->types[$block->get('nights')] = $block;
				}
			}
			else {
				$this->blocks[] = $block;
			}
		}
		
		return $this;
	}
	
	/**
	 * Method to add multiple blocks at once
	 *
	 * @access		public
	 * @param		array	$blocks 
	 * @return 		mixed	false on nothing found | Bregistrty price otherwise	
	 * @since		1.3.2
	 */
	public function addBlocks($blocks)
	{
		foreach ( $blocks as $block ) {
			$this->addBlock($block);
		}
		
		return $this;
	}
	
	/**
	 * Method to get the active price in the calendar
	 * If no arrival/departure is set then we just return first possible value
	 *
	 * @access		public
	 * @param		none
	 * @return 		mixed	false on nothing found | Bregistrty price otherwise	
	 * @since		1.3.2
	 */
	public function getActivePrice()
	{
		$return = false;
		
		if ( count($this->prices) ) {		
			if ( ($arrival = $this->search->get('arrival')) && ($departure = $this->search->get('departure')) && isset($this->prices[$arrival.'-'.$departure]) ) {
				$return = $this->prices[$arrival.'-'.$departure];
			}
			else {
				$return = $this->app->array->first($this->prices);
				$this->search->set('arrival', $return->get('arrival'));
				$this->search->set('departure', $return->get('departure'));
			}
		}
		
		return $return;
	}
	
	
	/**
	 * Method to get the active month in the calendar
	 *
	 * @access		public
	 * @param		none
	 * @return 		string	month of activeprice or current month	
	 * @since		1.3.2
	 */
	public function getActiveMonth()
	{
		$activeprice = $this->getActivePrice();
		$return		 = $activeprice 
					 ? $this->app->date->create( $activeprice->get('arrival') )->format('Y-m-01')
					 : $this->app->date->today()->format('Y-m-01');
		
		return $return;		
	}
		
	/**
	 * Get a html select box for all possible types (nights)
	 *
	 * @access		public
	 * @param		none
	 * @return 		string	a html select box with all possible nighttypes
	 * @since		1.3.2
	 */
	public function getTypeList()
	{
		$options = array();

		foreach ( $this->types as $nights => $type ) 
		{
			$text 	= $type->get('week') 
					? $type->get('specialname', $type->get('periodname'))
					: $type->get('periodname');

			$options[$nights] = $this->app->html->select->option($nights, $text);
		}
		
		ksort($options);
		
		$activeprice = $this->getActivePrice();
		$selected 	 = $activeprice ? $activeprice->get('nights') : '';
		
		$default 	 = array($this->app->html->select->option('', BText::_('WIDGET_CALENDAR_LABEL_TYPE')));
		$options 	 = array_merge($default, $options);
		
		return $this->app->html->select->genericlist('pricetype', $options, $selected, 'data-type class="bz-input bz-width-1-1"');
	}
	
	/* 
	 * Build a list of possible departures
	 *
	 * @param	none
	 * @param	date	$active			the active departuredate
	 */
	public function getDepartureList()
	{
		$activeprice = $this->getActivePrice();
		$dateformat  = $this->app->config->get('template.book_date_notation', 'DATE_FORMAT_DEFAULT');
		
		$selected 	 = '';
		$departures  = array();
		$options 	 = array();
		
		if ( $activeprice ) {
			$selected = $activeprice->get('departurestamp');
			
			foreach ( $this->prices as $price ) {
				if ( $price->get('arrival') == $activeprice->get('arrival') ) {
					$options[] = $this->app->html->select->option(
						$price->get('departurestamp'), $this->app->date->create($price->get('departure'))->format($dateformat)
					);
				}
			}
		}
				
		// create list
		return $this->app->html->select->genericlist('departure', $options, $selected, 'class="bz-input bz-size-small"');
	}
	
	/* 
	 * Build a list of possible departures
	 *
	 * @param	none
	 * @param	date	$active			the active departuredate
	 */
	public function getAlternativePrices()
	{
		$return 	 = array();

		if ( $activeprice = $this->getActivePrice() ) {	
			foreach ( $this->prices as $price ) {
				if ( $price->get('arrival') == $activeprice->get('arrival') ) {
					$return[] = $price;
				}
			}
		}
			
		// create list
		return $return;
	}
	
	/**
	 * Render the calendar
	 * This will render all months available
	 *
	 * @access		public
	 * @param		boolean	$echo	if true it will print the html at once on false it will return the html
	 * @return 		mixed			html on echo is true, void otherwise
	 * @since		1.3.2
	 */
	public function render($echo = false)
	{
		// first get correct months
		$min	= $this->app->date->create()->format('Y-m-d');
		$max	= $this->app->date->create()->add(1,'y')->format('Y-m-d');
		$months = array();
		
		if ( $this->settings->get('emptybegin',1) && $minprice = $this->app->array->first($this->prices) ) {
			$min = $this->app->date->create($minprice->get('arrival'))->format('Y-m-01');
		}
		
		if ( !$this->settings->get('maxmonths',12) && $maxprice = $this->app->array->last($this->prices) ) {
			$max = $this->app->date->create($maxprice->get('departure'))->format('Y-m-01');
		}
		
		while ( $min <= $max ) {
			$months[] = $min;
			$min = $this->app->date->create($min)->add(1,'m')->format('Y-m-01');
		}
		
		// get other needed stuff
		$dayclasses		= $this->getDayClasses();
		$return 		= array();
						
		// loop months building html
		foreach( $months as $month ) 
		{
			$tmp 		= new stdClass();
			$tmp->month = $month;
			
			// open table
			$html 	= array();
			$class  = array('bz-calendar-table');
			if ( $this->settings->get('showprices') ) {
				$class[] = 'with-prices';
			}
			if ( $this->settings->get('showweeknr', 1) ) {
				$class[] = 'with-weeks';
			}
			$html[] = '<table class="'.implode(' ', $class).'">';
			
			// add header
			$html[]	= $this->addHeader($month);
			
			// add days
			if ( $this->settings->get('show_days', 1) ) 
			{
				$html[]	= $this->addDays();
			}			
			
			// add body
			$html[]	= $this->addBody($month, $dayclasses);
			
			// close table
			$html[]	= '</table>';
			
			$tmp->html = implode("\n", $html);
			
			// glue
			$return[] = $tmp;
		}
		
		return $return;
	}
	
	
	/**
	 * This function creates a row by calling addCell for each cell
	 *
	 * @access		private
	 * @param		$cells		array	array with all the cells for a specific row
	 * @return 		string
	 * @since		1.0
	 */
	protected function row( Array $cells = array(), $attr = '') 
	{
		$html 	= array();
		$class	= '';
		
		// get class
		if ( isset($attr['class']) ) {
			$class = $this->app->array->implodeRecursive($attr['class'], ' ', true);
			
			if ( strlen($class) ) {
				$class = 'class="'.$class.'"';
			}
		}
		
		// initiate
        $html[]	= '<tr '.$class.'>';
        
		// add cells
        foreach ($cells as $cell) {
			$html[] = $cell;      
        }
      
	  	// close row 
        $html[] = '</tr>';
		
		// return
		return implode("\n", $html);		
	}
	
	/**
	 * This function creates a cell and adds aditional html if requested
	 *
	 * @access		private
	 * @param		$cell 	array	a cell with the value and additional html
	 * @param		$type	string	the type of cell td or th
	 * @return 		string	
	 * @since		1.0
	 */
	protected function cell($text = '', $attr = '', $type = 'td') 
	{	
		$type 		= strtolower($type) == 'th' ? 'th' : 'td';
		$class		= '';
		$attributes	= '';
		
		if ( isset($attr['class']) ) {
			$class = $this->app->array->implodeRecursive($attr['class'], ' ', true);
			
			if ( strlen($class) ) {
				$class = 'class="'.$class.'"';
			}
		}
		
		if ( isset($attr['attr']) ) {
			$attributes = $this->app->array->implodeRecursive($attr['attr'], ' ', true);
		}
			
		return '<'.$type.' ' . $class . ' ' . $attributes . '>' .$text. '</'.$type.'>';
	}
		
	/**
	 * This function adds all the rows with the days for a requested month
	 * It will check each day for availabilty and add a class to the cell (day)
	 * We start off by getting the nr of weeks and nr of days for a month
	 * We add a correction to the starting day if calendar starts with a monday
	 * All days get calculated by counter + nr of days (i) - starting day
	 * This will make a day in last month get a value of 0 or below.
	 * We correct that value by adding up the nr of days of last month
	 *
	 * @access		private
	 * @param		none
	 * @return 		string	
	 * @since		1.0
	 */
	private function addBody($month, $dayclasses)
	{
		// initiate
		$body		= array();
		
		// defaults
		$counter	= 1;
		$monthstamp	= strtotime($month);
		$monthshort = date('Y-m', $monthstamp);		
		
		// set some values that we are gonna need
		$today			= $this->app->date->create()->format('Y-m-d');
		$dayscount		= date('t',$monthstamp);
		$startday		= (int) date('w',$monthstamp);
		$dayslastmonth 	= date('t',$monthstamp - 86400);
		$weekcount 		= !$this->settings->get('showweeknr', 0) 
						? 6
						: ceil( ($dayscount + $startday ) / 7 ); 
		$startday		= $this->settings->get('sunday', 0) 
						? $startday 
						: ( $startday ? $startday - 1 : $startday + 6);	

		// loop the weeks
		for( $j = 0; $j < $weekcount; $j++ ) 
		{
			// initiate row	
			$row		= array();
		
			// do we add the weeknr
			if ( $this->settings->get('showweeknr', 1) ) 
			{
				$weeknr = date('W', mktime(0,0,0,date('n',$monthstamp),$counter,date('Y',$monthstamp)));
				$row[] 	= $this->cell($weeknr, array('class' => 'weeknr'));
			}
			
			// add 7 days for each week
			for ( $i = 0; $i <= 6; $i++ ) 
			{
				// get day
				$day	= ( $i + $counter ) - $startday;
				
				// day is in other month
				if ( $day < 1 || $day > $dayscount ) 
				{
					$day	= $day < 1 ? (int) $day + $dayslastmonth : (int) $day - $dayscount;
					$row[] 	= $this->cell($day, array('class' => 'other-month'));
					continue;
				}
				
				$daydate	= $this->app->date->create($monthshort .'-'.$day);
				$date		= $daydate->format('Y-m-d');	
				$stamp  	= $daydate->toUnix();
				
				// day is in this month but ran out of date
				if ( $date < $today && $oldclass = $this->settings->get('old') ) {
					$row[] 	= $this->cell($day, array('class' => $oldclass));
					continue;
				}
				
				// active cell
				$dayclass = array_key_exists($date, $dayclasses) ? $dayclasses[$date] : ($this->settings->get('bookedbydefault') ? 'booked' : 'free');
				
				
				$row[] = $this->cell($day, array('class' => $dayclass, 'attr' => 'data-stamp="'.$stamp.'" data-day="'.$day.'"'));
			}
			
			// add the week to the row and add a week to the counter
			$body[]	  = $this->row($row, array('class' => 'dates'));
			$counter += 7;
		}	
		
		return implode("\n", $body);
	}
	

	
	/**
	 * Gets a list of classes for each day
	 *
	 * @access		private
	 * @param		$day	string	the date for a day
	 * @param		$days	array	all days know so far
	 * @return 		$days	array	a day with its class	
	 * @since		1.0
	 */
	private function getDayClasses( )
	{
		$dates			= array();
		$blocks 		= $this->blocks;
		$bookeddefault 	= $this->settings->get('bookedbydefault', 1);
		
		// if booked by default let prices be blocks 
		if ( $bookeddefault ) {
			
			$blocks = array();
			$prices = $this->prices;
			
			if ( count($prices) ) {
				$first	  = $this->app->array->first($prices)->toArray();
				$blocks[] = $this->app->registry->create($first);

				// merge prices into one block
				foreach ( $prices as $price ) {
					$c = count($blocks)-1;
					if ( $price->get('arrival') <= $blocks[$c]->get('departure') ) {
						$blocks[$c]->set('departure', $price->get('departure'));
					} 		
					else {
						$blocks[] = $this->app->registry->create($price->toArray());
					}
				}
			}
		}

		foreach ( $blocks as $block ) 
		{
			$arrival = $block->get('arrival');

			while( $arrival <= $block->get('departure') ) {
				if ( $arrival == $block->get('arrival') ) {
					$dates[$arrival] = $bookeddefault ? 'free free-start' : 'free free-end';
				}
				elseif ( $arrival == $block->get('departure') ) { 
					$dates[$arrival] = $bookeddefault ? 'free free-end' : 'free free-start';
				}
				else {
					$dates[$arrival] = $bookeddefault ? 'free' : 'booked';
				}

				$arrival = $this->app->date->create($arrival)->add(1)->format('Y-m-d');
			}
		}	
		
		return $dates;
		
	}

	
	protected function addHeader($month)
	{
		$headerrow		= array();
				
		$headerrow[] 	= $this->cell(
								$this->app->date->create($month)->format('F Y'), 
								array('class' => 'month', 'attr' => 'colspan="'.($this->settings->get('showweeknr', 1) ? 8 : 7).'"'), 'th'
							);		
		
		return $this->row($headerrow, array('class' => 'month'));
	}
	
	protected function addDays()
	{
		$daysrow	= array();
		
		if ( $this->settings->get('showweeknr', 1) ) {
			$daysrow[] = $this->cell();
		}
		
		$days		= $this->settings->get('sunday', 0)
					? array('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat')
					: array('mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun');
									
		foreach ( $days as $day ) {
			$daysrow[]	= $this->cell(BText::_($day), array('class' => 'day'));
		}
		
		return $this->row($daysrow, array('class' => 'days'));
	}
	
	public function getPriceData()
	{
		$dates 		= array();
		
		foreach ( $this->prices as $price ) 
		{ 
			$date 				= new stdClass();
			$date->arrival 		= $price->get('arrivalstamp');
			$date->departure 	= $price->get('departurestamp');
			$date->price		= $price->get('pricebeforediscount');
			$date->nights		= $price->get('nights');
			$date->name			= $price->get('specialname', $price->get('periodname'));
			if ( $price->get('discount') ) {
				$date->discount	= $price->get('discount');
			}
			if ( $extra = $price->get('extracosts') ) {
				$date->costs		= $extra;
			}
			/*if ( $price->get('arrangementid') && $arrangementname = $price->get('originalname')) { 
				$date->arrangement 	= $arrangementname;
			}*/
					
			//$dates[] = $this->app->registry->create($date)->toString();
			$dates[] = $date;
		}
		
		return $dates;
	}
	
	public function getDataString()
	{
		$datastring 	= $this->app->registry->create();
		$periodstring 	= $this->app->registry->create();
		$active		 	= $this->getActivePrice();
		$dates 			= $this->getPriceData();
		$oid			= $this->search->get('id');
		$aid 			= $this->search->get('aid');
		$pets			= $this->search->get('pets');
		$persons		= $this->search->get('persons');
		
		$datastring->set('prices', 			$dates);
		$datastring->set('type',			$this->settings->get('type','none'));
		$datastring->set('action',			(bool) $this->settings->get('action', 1));
		$datastring->set('showprices',		(bool) $this->settings->get('showprices', 0));
		$datastring->set('show',			$this->settings->get('months',2));
		$datastring->set('priceformat',		$this->settings->get('pricestyle', $this->app->config->get('template.pricestyle', 'always')));
		$datastring->set('calcpriceformat',	$this->settings->get('pricecalculationpricestyle', $this->app->config->get('template.pricestyle', 'always')));
		$datastring->set('priceincl',		(bool) $this->app->config->get('template.pricesincl', 0));
		$datastring->set('dateformat', 		BText::_($this->app->config->get('template.book_date_notation', 'DATE_FORMAT_DEFAULT')));
		$datastring->set('dateshort',		BText::_('WIDGET_CALENDAR_DATE_ALTERNATIVES'));
		$datastring->set('baselink', 		$this->app->price->booklink(array(), array('objectid'=> $oid, 'persons' => $persons, 'pets' => $pets, 'arrangementid' => $aid)));
		$datastring->set('calculation', 	!in_array($this->settings->get('pricecalculation'), array(0,4)) );
		$datastring->set('pricelist', 		$this->settings->get('pricecalculation') ? (bool) $this->settings->get('showpricelist') : false);
		$datastring->set('alternatives',	$this->settings->get('pricecalculation') == 4 || ($datastring->get('calculation') && $this->settings->get('departurechange',0) == 2));
		$datastring->set('alternativeactive', $this->settings->get('pricecalculation') == 4);
		if ( $this->settings->get('type') != 'none' && $this->settings->get('action') ) {
			$datastring->set('highlight',		(int) $this->settings->get('highlight', 1));
		}
		$datastring->set('oid', 			$oid);
		
		if ( $active ) {
			$periodstring->set('arrival', $this->app->date->create($active->get('arrival'))->toUnix());
			$periodstring->set('departure', $this->app->date->create($active->get('departure'))->toUnix());
			
			if ( $this->settings->get('highlight', 1) == 2 ) {
				$datastring->set('highlightstart', 1);
			}
		}
		
		
		// add javascript to header
		BHtml::_('behavior.calendar');
			
			
		return ' data-bz-calendar=\''.$datastring->toString().'\' data-period=\''.$periodstring->toString().'\' ';
	}
}