<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperApi extends BHelper {

	public $request;
	
	public $query;
	
	public $generalcachetime;
	
	public $periodscachetime;
	
	public function __construct($app)
	{
		parent::__construct($app);
		
		$this->request 			= BookzoApiRequest::getInstance();
		$this->query 			= $this->request->getQuery(true);
		$this->generalcachetime = (int) $this->app->config->get('general.general_cache', 15);
		$this->periodscachetime = (int) $this->app->config->get('general.periods_cache', 1);
	}
	
	/**
	 * Method to get the bookzo config settings
	 * 
	 * @param 	none
	 * @return 	return	array a list of settings
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getSettings()
	{
		if ( !isset($this->cache['settings']) ) 
		{		
			$result 		= $this->getResult('GetSettings');

			$this->cache['settings'] = $result;
		}
		
		return $this->cache['settings'];
	}
	
	/**
	 * Method to get a list of countries from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getCountries( $ids = null )
	{
		// get groups just once
		if ( !isset($this->cache['countries']) ) 
		{
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetCountries', array(), 'id', array())
			);
						
			// store converted into cache			
			$this->cache['countries'] 	= $this->getResponseData($results);
		}
		
		return $this->app->array->returnMixed($this->cache['countries'], $ids);				
	}
	
	/**
	 * Method to get a list of languages from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getLanguages( $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['languages']) ) 
		{
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetLanguages', array(), 'id', array())
			);
						
			// store converted into cache			
			$this->cache['languages'] 	= $this->getResponseData($results);
		}
		
		return $this->app->array->returnMixed($this->cache['languages'], $ids);		
	}
	
	/**
	 * Method to get a list of extra cost types
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getExtraCosts( $ids = null )
	{
		if ( !isset($this->cache['extracosts']) ) 
		{			
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetExtraCosts', array(), 'id', array())
			);
			
			// store converted into cache			
			$this->cache['extracosts'] 	= $this->getResponseData($results, 'BookzoResponseData', 'id');
		}
		
		return $this->app->array->returnMixed($this->cache['extracosts'], $ids);	
	}
	
	/**
	 * Method to get a list of paymentmethods from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getPaymentMethods( $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['paymentmethods']) ) 
		{			
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('Bookkeeping/GetPaymentMethods', array(), 'id', array())
			);
			
			// store converted into cache			
			$this->cache['paymentmethods'] 	= $this->getResponseData($results);
		}
		
		return $this->app->array->returnMixed($this->cache['paymentmethods'], $ids);		
	}
	
	/**
	 * Method to get a list of pricedefinitions (day, week etc)
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getPriceDefinitions( $ids = null )
	{
		if ( !isset($this->cache['pricedefinitions']) ) 
		{
			// set default data
			$query	= $this->app->registry->create();
			$query->set('From', 	$this->app->date->today()->format('Y-m-d'));
			$query->set('Until', 	$this->app->date->nextyear()->format('Y-12-31'));
						
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetPriceDefinitions', $query, 'id', array(), true, null, 'queries')
			);
			
			// get converted data			
			$definitions 	= $this->getResponseData($results);
					
			// sort them by nrofnights
			uasort($definitions, function($a, $b) { return strcmp($a->numberofnights, $b->numberofnights); });

			// store in cache
			$this->cache['pricedefinitions'] = $definitions;
		}
		
		return $this->app->array->returnMixed($this->cache['pricedefinitions'], $ids, true);
	}
	
	/**
	 * Method to get a list of actions from bookzo
	 *
	 * @param 	active	boolean	do the actions have to be active 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getActions( $active = false, $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['actions']) ) 
		{			
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetActions', array(), 'id', array())
			);
			
			// store converted into cache			
			$this->cache['actions'] 	= $this->getResponseData($results, 'BookzoAction');			
		}
		
		$actions = $this->cache['actions'];
		
		// do we want only active actions
		if ( $active && is_null($ids) )
		{
			$today 	 = $this->app->date->create()->format('Y-m-d');
			
			foreach ( $actions as $k => $action ) {
				$frombookable 	= $action->getDate('frombookingdate');
				$untilbookable 	= $action->getDate('untilbookingdate');
      			$untilarrival 	= $action->getDate('untilarrivaldate');
				
				if ( $today < $frombookable || $today > $untilbookable || $today > $untilarrival || $action->get('onlyforage') || $action->get('onlywithcouponcode') ) {
					unset($actions[$k]);
				}
			}
		}
		
		return $this->app->array->returnMixed($actions, $ids);			
	}
	
	/**
	 * Method to get a list of arrangements from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getArrangements( $ids = null )
	{
		// get groups just once
		if ( !isset($this->cache['arrangements']) ) 
		{
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetArrangements', array(), 'id', array())
			);
						
			// store converted into cache			
			$this->cache['arrangements'] 	= $this->getResponseData($results, 'BookzoArrangement');
		}
		
		return $this->app->array->returnMixed($this->cache['arrangements'], $ids);				
	}
	
	/**
	 * Method to get a list of Filters from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFilters( $ids = null )
	{
		// get filters just once
		if ( !isset($this->cache['filters']) ) 
		{
			$results 	= $this->getResults('getFilterFeatures', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);
			$filters	= array();					
			$i 			= 0;
			
			foreach ( $results as $result ) 
			{
				$group	 	= new BookzoResponseData($result);
				$groupname = $group->translate('name');
				
				foreach ( $group->features as $filter ) {					
					if ( (int) $filter->useforfilter == 1 ) {
						$filters[$filter->id] = new BookzoFiltersFilter($filter);
						$filters[$filter->id]->set('groupname', $groupname);
						$i++;
					}
				}
			}
			
			$this->cache['filters'] = $filters;
		}
		
		return $this->app->array->returnMixed($this->cache['filters'], $ids);		
	}
	
	/**
	 * Method to get a list of Features from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFeatures( $ids = null )
	{
		// get features just once
		if ( !isset($this->cache['features']) ) 
		{
			$results 	= $this->getResults('getFeatures', array(), 'id', array());	
			$results 	= $this->app->array->objectKeysToArray($results);			
			$list 		= array();			
			
			foreach ( $results as $result ) 
			{
				$groupname = $result->name;
				
				if ( isset($result->texts) && is_array($result->texts) ) {
					$lang 	= $this->app->language->getId();
					
					foreach ( $result->texts as $language ) 
					{
						if ( $language->languageid == $lang && isset($language->name) && !empty($language->name) ) {
							$groupname = $language->name;
							break;
						}
					}
				}
				
				foreach ( $result->features as $features ) {
					$id				   		= $features->id;
					$features->groupname 	= $groupname;
					
					$list[$id] = new BookzoFeaturesFeature($features);
				}
			}
			
			$this->cache['features'] = $list;
		}
		
		return $this->app->array->returnMixed($this->cache['features'], $ids);		
	}
	
	/**
	 * Method to get a list of groups from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getGroups( $ids = null )
	{
		// get groups just once
		if ( !isset($this->cache['groups']) ) 
		{			
			// get results
			$results = $this->app->array->objectKeysToArray(
				$this->getResults('GetGroups', array(), 'id', array())
			);
			
			// store converted into cache			
			$this->cache['groups'] 	= $this->getResponseData($results, 'BookzoGroup', 'id');
		}
		
		return $this->app->array->returnMixed($this->cache['groups'], $ids);		
	}
	
	/**
	 * Method to get a list of (or single) object(s) from bookzo
	 * 
	 * @param 	ids		array	an id or a list of ids to return
	 * @return 	return	array	an array of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getObjects( $ids = null ) 
	{
		// get object just once
		if ( !isset($this->cache['objects']) ) 
		{		
			if ( !$results = $this->app->cache->get('objects', $this->generalcachetime ) ) 
			{				
				$this->app->session->clear('bookzo.randomids');
			
				// get total number of accommodations
				$total		= $this->app->config->get('bookzo.numberofaccommodations', 0);
				$loops 		= ceil($total/10);
				$fetched	= array();			
				$results	= array();

				// loop until maxloops reached
				for ( $i = 0; $i < $loops; $i++ ) 
				{
					$fetched = $this->getResults( 'GetObjects', array('Page' => $i), null, array(), false, null, 'queries', true);
					
					if ( is_array($fetched) && count($fetched) )
					{
						$results = array_merge($results, $fetched);
					}
					// no need to proceed on error
					elseif ( $i != $loops ) {
						break;
					}
				}
				
				// get from cache
				$cachedresults 		  = $this->app->cache->get('objects', 0);
				$cachedresultsdecoded = json_decode($cachedresults);
				
				// if results doesn't match total we take te cache
				if ( count($results) == $total || count($results) > count($cachedresultsdecoded) )
				{
					$this->app->cache->cache('objects', json_encode($results));
				}
				else {
					//$results = $this->app->cache->get('objects', 0);
					$this->app->cache->cache('objects', $cachedresults);
					//$results = json_decode($results);
					$results = $this->app->array->objectKeysToArray($cachedresultsdecoded);
				}
			}
			else {

				$results = json_decode($results);
				$results = $this->app->array->objectKeysToArray($results);
			}

			$objects = array();
			
			foreach ( $results as $result ) 
			{
				if ( isset($result->id) ) {
					$objects[$result->id] = new BookzoObject($result);
				}
			}			

			$this->cache['objects'] = $objects;
		}
		
		return $this->app->array->returnMixed($this->cache['objects'], $ids);				
	}
	
	/**
	 * Method to get all objects correlated to a specific group
	 * 
	 * @param 	id		groupid		the group to get the objects from
	 * @return 	return	array		a list of objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getObjectsByGroup( $groupid )
	{
		$objects = $this->getObjects();
		$return  = array();
		
		if ( !( (int) $groupid ) ) {
			return $objects;
		}

		foreach ( $objects as $k => $object ) 
		{
			if ( $object->get('groupid') == $groupid ) {
				$return[$k] = $object;
			}
		}
		
		return $return;
	}
	
	/**
	 * Method to get a list of free object within a certain timeframe from bookzo
	 * 
	 * @param 	vars		array	a list of parameters
	 * @return 	return		array	an array of BookzoObject objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFreeObjects($vars) 
	{
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		
		// build vars
		$query->set('Arrival', 			$this->app->date->create($vars->get('arrival'))->format('Y-m-d'));
		$query->set('Departure',		$this->app->date->create($vars->get('departure'))->format('Y-m-d'));	
		$query->set('GroupId',			$vars->get('group', 0));
		$query->set('NumberOfPersons',	$vars->get('persons', 0));
		$query->set('NumberOfPets',		$vars->get('pets', 0));
		$query->set('Arrangements',		$vars->get('arrangement', $vars->get('arrangement')));
		
		if ( $query->get('Arrival') < $this->app->date->create()->format('Y-m-d')) {
			return array();
		}
		
		// clean query and create key
		$unique	= $query->clean(true)->createkey();

		// get free objects just once
		if ( !isset($this->cache['freeobjects'][$unique]) ) 
		{		
			$results 	= $this->getResults('GetFreeObjects', $query, 'objectid', array(), true, $this->periodscachetime, 'periodqueries' );
			$results 	= $this->app->array->objectKeysToArray($results);
			$return		= array();

			// modify in correct format
			foreach ( $results as $result ) {
				$tmp 			= new stdClass();
				$tmp->objectid 	= $result->objectid;
				$tmp->price		= $this->app->price->create( $result, array('arrival' => $query->get('Arrival'), 'departure' => $query->get('Departure'))); 
				
				$return[$result->objectid] = $tmp;
			}

			$this->cache['freeobjects'][$unique] = $return;
		}
		
		return $this->cache['freeobjects'][$unique];
	}
	
	/**
	 * Method to get a list of available months periods
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of available months
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getAvailableMonths($vars)
	{
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		
		// build vars
		$query->set('Group',		$vars->get('group', 0));
		$query->set('Persons',		$vars->get('persons', 0));
		$query->set('Pets',			$vars->get('pets', 0));
		$query->set('Arrangements', $vars->get('arrangements', $vars->get('arrangement')));

		// clean query and create key
		$unique	= $query->clean(true)->createkey();
		
		// fetch if not fetched allready
		if ( !isset($this->cache['availablemonths'][$unique]) ) 
		{		
			$results 	= $this->getResults('GetBookableMonths', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );

			## NOTE - For now bookzo does not return sorted results
			sort($results);

			$this->cache['availablemonths'][$unique] = $results;
		}
		
		return $this->cache['availablemonths'][$unique];
	}
	
	/**
	 * Method to get a list of available arrivals
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of available arrivals
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getArrivals($vars, $onlyfirst = false)
	{	
		// build correct variables
		$vars  		= $this->app->registry->create( (array) $vars );
		$query		= $this->app->registry->create();
		$today 		= $this->app->date->create()->format('Y-m-d');
		$oneyear	= 
		$priceid 	= $vars->get('priceid', 0);
		
		// set dates
		$query->set('From', 	$this->app->date->create($vars->get('from'))->format('Y-m-d'));
		if ( $today > $query->get('From') ) {
			$query->set('From', $today);
		}
		$query->set('Until', 	$this->app->date->create($vars->get('until', $this->app->date->create($query->get('From'))->add(1,'year')->format('Y-m-d')))->format('Y-m-d'));
				
		// object or group
		if ( (int) $vars->get('object') ) {
			$query->set('Object', (int) $vars->get('object'));
		}
		else {
			$query->set('Group',	$vars->get('group', 0));
			$query->set('Persons',	$vars->get('persons', 0));
			$query->set('Pets',		$vars->get('pets', 0));
		}
		
		// arrangements
		$query->set('Arrangements',	$vars->get('arrangements', $vars->get('arrangement')));

		// clean query and create key
		$unique	= $query->clean(true)->createkey();
		
		// do it just once
		if ( !isset($this->cache['arrivals'][$unique]) ) 
		{			

			## NOTE - We merge priceids but this is kinda silly
			## ASK Jon to add more then 1 priceid in request method
			$tmp		= array();
			$priceid	= explode('-', $priceid);
			foreach( $priceid as $v ) {
				if ( $v ) {
					$query->set('PriceId', $v);
				}
				
				$results 	= $this->getResults('GetPossibleArrivals', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );
				$arrivals  	= array();
				foreach ( $results as $a ) {
					$arrivals[] = $this->app->date->create($a)->format('Y-m-d');
				}
				
				$tmp = array_unique(array_merge($tmp, $arrivals));
				
			}
			
			## NOTE - For now bookzo does not return sorted results
			sort($tmp);
			
			$this->cache['arrivals'][$unique] = $tmp;
		}
		
		if ( $onlyfirst ) {
			return $this->app->array->first($this->cache['arrivals'][$unique]);
		}
		
		return $this->cache['arrivals'][$unique];
	}
	
	/**
	 * Method to get a list of available departures
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of available departures
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getDepartures($vars, $onlyfirst = false)
	{
		// build correct variables
		$vars  		= $this->app->registry->create( (array) $vars );
		$query		= $this->app->registry->create();
		
		// set dates
		$query->set('From', 	$this->app->date->create($vars->get('from', $vars->get('arrival')))->format('Y-m-d'));
				
		// object or group
		if ( (int) $vars->get('object') ) {
			$query->set('Object', (int) $vars->get('object'));
		}
		else {
			$query->set('Group',	$vars->get('group', 0));
			$query->set('Persons',	$vars->get('persons', 0));
			$query->set('Pets',		$vars->get('pets', 0));
		}
		
		// arrangements
		$query->set('Arrangements',	$vars->get('arrangements', $vars->get('arrangement')));
		
		$unique	= $query->clean(true)->createkey();

		// do it just once
		if ( !isset($this->cache['departures'][$unique]) ) 
		{		
			if ( !$query->count() ) {
				$this->cache['departures'][$unique] = array();
			}
			else {
		
				$results 	= $this->getResults('GetPossibleDepartures', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );

				$departures = array();
				foreach ( $results as $a ) {
					$departures[] = $this->app->date->create($a)->format('Y-m-d');
				}
				
				## NOTE - For now bookzo does not return sorted results		
				sort($departures);
				
				$this->cache['departures'][$unique] = $departures;
			}
		}

		$return = $this->cache['departures'][$unique];
		
		if ( $onlyfirst ) {
			return $this->app->array->first($this->cache['departures'][$unique]);
		}
		
		return $this->cache['departures'][$unique];
	}
	
	/**
	 * Method to get a list of bookable periods
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of bookable periods
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getBookablePeriods($vars)
	{
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		
		$query->set('From', 	$this->app->date->create($vars->get('from'))->format('Y-m-d'));
		$query->set('Until', 	$this->app->date->create($vars->get('until'))->format('Y-m-d'));
		$query->set('Group',	$vars->get('group', 0));
		$query->set('Persons',	$vars->get('persons', 0));
		$query->set('Pets',		$vars->get('pets', 0));
		
		// clean query and create key
		$unique	= $query->clean(true)->createkey();
		
		// do it just once
		if ( !isset($this->cache['bookableperiods'][$unique]) ) 
		{
			$results 	= $this->getResults('GetBookablePeriods', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );
			$results 	= $this->app->array->objectKeysToArray($results);
			
			$this->cache['bookableperiods'][$unique] = $results;
		}
		
		// return
		return $this->cache['bookableperiods'][$unique];
	}
	
	/**
	 * Method to get a list of booked periods
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of bookable periods
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getBooked($vars)
	{
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		
		$query->set('From', 		$this->app->date->create($vars->get('from'))->format('Y-m-d'));
		$query->set('Until', 		$this->app->date->create($vars->get('until'))->format('Y-m-d'));
		$query->set('ObjectId',		$vars->get('objectid', 0));
		$query->set('ConcatPeriods','true');
		
		// clean query and create key
		$unique	= $query->clean(true)->createkey();
		
		// do it just once
		if ( !isset($this->cache['bookedperiods'][$unique]) ) 
		{
			$results 	= $this->getResults('GetBookedPeriods', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );
			$return 	= array();
		
			foreach ( $results as $result ) {
				$tmp				= new stdClass();
				$tmp->arrival 		= $this->app->date->create($result->arrival)->format('Y-m-d');
				$tmp->departure 	= $this->app->date->create($result->departure)->format('Y-m-d');
				$return[$tmp->arrival .'-'.$tmp->departure] = $tmp;
			}
			
			ksort($return);
	
			## NOTE - we need to concat ourself bookzo doesn't do this properly
			## Ask Jon to look into this
			$tmp 	= array_values($return);
			$return = array();
			$c 	 	= 0;
			
			for( $k = 0; $k < count($tmp); $k++ ) 
			{
				$return[$c]				= new stdClass();
				$return[$c]->arrival 	= $tmp[$k]->arrival;
				$return[$c]->departure 	= $tmp[$k]->departure;
				
				if ( $k < count($tmp) - 1 ) {
					for ( $i = $k + 1; $i < count($tmp); $i++ ) {
						if ( $tmp[$i]->arrival == $return[$c]->departure ) {
							$return[$c]->departure = $tmp[$i]->departure;
							$k++;
						}
						else {
							break;
						}
					}
				}
				
				$c++;
			}
			
			$this->cache['bookedperiods'][$unique] = $return;
		}
		
		return $this->cache['bookedperiods'][$unique];
	}
	
	/**
	 * Method to get a list of available priceperiods
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of bookable periods
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getPrices($vars, $divide = false)
	{	
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		$method = 'GetFreePeriods';
		
		$query->set('From', 		$this->app->date->create($vars->get('from', $this->app->date->today('Y-m-d')))->format('Y-m-d'));
		$query->set('Until', 		$this->app->date->create($vars->get('until', $this->app->date->nextyear('Y-m-d')))->format('Y-m-d'));
		$query->set('Persons',		$vars->get('persons', 2));
		$query->set('Pets',			$vars->get('pets', 0));
		$query->set('Arrangements',	$vars->get('arrangements', $vars->get('arrangement', 0)));
		$query->set('Permutated', 	1);

		// object or group
		if ( $objectid = $vars->get('objectid', 0) ) {
			$query->set('ObjectId', $objectid);			
		}
		else if ( $groupid = $vars->get('group', 0) ) {
			$query->set('Group', $groupid );
		}

		// add actions
		if ( $actions = (array) $vars->get('actions') ) {
			$query->set('Actions', is_array($actions) ? implode(',', $actions) : (int) $actions); 		
		}

		// make unique string for caching
		$unique	  = $query->clean(true)->createkey();

		// do it just once
		if ( !isset($this->cache['prices'][$unique]) ) 
		{
			$results 		= $this->getResults($method, $query, null, array(), true, $this->periodscachetime, 'periodqueries' );

			$return 		= array();
			
			foreach ( $results as $result ) {
				foreach ( $result->departures as $departure ) {
					$departure->arrival = $result->arrival;
					if ( $divide ) {
						$return[$departure->objectid][] = $departure;
					}
					else {
						$return[] = $departure;
					}
				}
			}
			
			// lets order 
			if ( $divide ) {
				foreach ( $return as $k => $r ) {
					$return[$k] = $this->app->periods->sort($r);
				}
			}
			else {
				$return = $this->app->periods->sort($return);
			}

			// add to internal cache
			$this->cache['prices'][$unique] = $return;
		}
		
		// return
		return $this->cache['prices'][$unique];
	}
	
	/**
	 * Method to get a prospect booking
	 * 
	 * @param 	vars	array	a list of parameters
	 * @return 	return	array	a list of bookable periods
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getProspect($data, $log = true)
	{
		$data   = $this->app->registry->create( (array) $data );
		$vars	= $this->app->registry->create();
		
		// set default vars
		$vars->set('Arrival', 		$this->app->date->create($data->get('arrival'))->format('Y-m-d'));
		$vars->set('Departure', 	$this->app->date->create($data->get('departure'))->format('Y-m-d'));
		$vars->set('Groupid', 		$data->get('group', 0));
		$vars->set('ObjectId', 		$data->get('group', 0) ? 0 : $data->get('object', $data->get('oid')));
		$vars->set('ArrangementId', $data->get('arrangement', 0));
		$vars->set('Adults', 		$data->get('persons', $data->get('adults', 2)));
		$vars->set('Seniors', 		$data->get('seniors', 0));
		$vars->set('Childs', 		$data->get('childs', 0));
		$vars->set('Babies', 		$data->get('babies', 0));
		$vars->set('Pets', 			$data->get('pets', 0));
		$vars->set('CouponCode', 	$data->get('coupon'));
		$vars->set('LanguageId', 	$this->app->language->getId());
		
		if ( $vars->get('Arrival') < $this->app->date->create()->format('Y-m-d')) {
			return false;
		}
		
		// add items
		$items 		= (array) $data->get('items', array());
		$itemparts 	= array();
		foreach ( $items as $id => $amount )
		{
			if ( (int) $id && (int) $amount ) {
				$itemparts[] = (int) $id;
				$itemparts[] = (int) $amount;
			}
		}
		
		if ( count($itemparts) ) {
			$vars->set('Costs', implode(',', $itemparts));
		}
		
		// get unique key
		$unique	  = $vars->clean(true)->createkey();
		
		if ( !isset($this->cache['prospect'][$unique]) ) 
		{
			$result = false;
			
			if ( $vars->get('ObjectId') || $vars->get('GroupId') ) 
			{
				// try fetching the prospect	
				try {
					$this->query->clear();
					$this->query->set('GetReservationProspect');
					$this->query->wheres($vars->toArray());
	
					$result = $this->request->setQuery( $this->query )->getResult(true, $log);
				}
				catch ( BookzoRequestException $e ) {
					$result	= false;
				}
			}
			
			// set it into the internal cache
			$prospect = new BookzoProspect($result);
			$prospect->set('id', $unique);
			
			$this->cache['prospect'][$unique] = $prospect;
		}
		
		return $this->cache['prospect'][$unique];
	}
		 
	/*
	public function getProspect($oid, $arrival, $departure, $persons = 2, $seniors = 0, $childs = 0, $babys = 0, $pets = 0, $items = array(), $coupon = null, $group = 0, $log = true)
	{					
		// make unique string for caching
		## NOTE = also add in items
		$unique	  = base64_encode($oid.$arrival.$departure.$persons.$group);
		
		if ( !isset($this->cache['prospect'][$unique]) ) 
		{
			$result = false;
			
			if ( (int) $oid || (int) $group ) 
			{
				// build the vars
				$vars			 			= array();
				if ( $group ) {
					$vars['GroupId'] = $group;
				}
				else {
					$vars['ObjectId'] 			= (int) $oid;
				}
				$vars['Arrival'] 			= $this->app->date->create($arrival)->format('Y-m-d');
				$vars['Departure'] 			= $this->app->date->create($departure)->format('Y-m-d');
				$vars['Adults']				= (int) $persons;
				$vars['Seniors']			= (int) $seniors;
				$vars['Childs']				= (int) $childs;
				$vars['Babies']				= (int) $babys;
				$vars['Pets']				= (int) $pets;
				$vars['LanguageId']			= $this->app->language->getId();
				
				if ( strlen(trim($coupon)) ) {
					$vars['CouponCode'] = $coupon;
				}
							
				// do we add items
				$itemparts = array();
				foreach ( $items as $id => $amount )
				{
					if ( (int) $id && (int) $amount ) {
						$itemparts[] = (int) $id;
						$itemparts[] = (int) $amount;
					}
				}
				
				if ( count($itemparts) )
				{
					$vars['Costs'] = implode(',', $itemparts);
				}
				
				// try fetching the prospect	
				try {
					$this->query->clear();
					$this->query->set('GetReservationProspect');
					$this->query->wheres($vars);
	
					$result = $this->request->setQuery( $this->query )->getResult(true, $log);
				}
				catch ( BookzoRequestException $e ) {
					$result	= false;
				}
			}
			
			// set it into the internal cache
			$prospect = new BookzoProspect($result);
			$prospect->set('id', $unique);
			
			$this->cache['prospect'][$unique] = $prospect;
		}
		
		return $this->cache['prospect'][$unique];
	}
	*/
	
	public function book($fields)
	{
		$return = false;		
		$this->query->clear();
		$this->query->set('SubmitReservation');

		foreach ( $fields as $k => $v ) 
		{
			if ( strlen(trim($v)) ) {
				$this->query->where($k, $v);
			}
		}

		try {
			## NOTE - Set timeout to higher level for now until fix
			$this->request->setTimeout(15);
			$result = $this->request->setQuery( $this->query )->getResult();

			if ( is_object($result) && isset($result->arrival) ) {
				$return = $result;
			}
		}
		catch ( BookzoRequestException $e ) {
			$return	= false;
		}
		
		## NOTE - Reset timeout for now
		$this->request->setTimeout(5);
		
		return $return;
	}		

	
	public function addPayment( $trid, $amount, $desc, $eid = 0 )
	{
		$this->query->clear();
		$this->query->set('AddPaymentForTargetPay');
		
		$vars							= array();
		$vars['TargetpayTransactionId']	= $trid;
		$vars['PayDate']				= $this->app->date->create()->format('Y-m-d');
		$vars['Amount']					= number_format( (float) $amount, 2, '.', '');
		$vars['PaymentMethod']			= $this->app->config->get('bookzo.generatepaymentwhenpayedonline', 0) ? $this->app->config->get('pay.bookzopaymethod', 0) : 0;
		$vars['Description']			= $desc;
		$vars['ExternalId']				= $eid;
		
		$this->app->array->clearArray($vars, false);
		
		foreach ( $vars as $k => $v ) 
		{
			$this->query->where($k, $v);
		}
		
		try {
			$result = (bool) $this->request->setQuery( $this->query )->getResult( false, true );
		}
		catch ( BookzoRequestException $e ) {
			$return	= false;
		}
		
		return $return;
	}
	
	protected function getResults( $method, $vars = array(), $key = null, $default = false, $cache = true, $cachetime = null, $cachedir = 'queries', $returnfalse = false, $log = true )
	{
		// vars should be an array
		if ( $vars instanceof BRegistry ) {
			$vars = $vars->toArray();
		}
		
		// set default values
		$found = false;
		$old   = $default;
				
		// do we load from cache
		if ( $cache ) 
		{
			$cachetime = $cachetime ? $cachetime : $this->generalcachetime;
			$cachename = strtolower($method);
			
			if ( is_array($vars) && count($vars) )
			{
				$cachename .= '.' . md5(http_build_query($vars));
			}
			
			// do we have a valid cache
			if ( $return = $this->app->cache->get($cachename, $cachetime, $cachedir) )
			{
				return json_decode($return);
			}
			else {
				// load old cache if api does not return anything. 
				// we can return old data to not get blank lists
				if ( $cached = $this->app->cache->get($cachename, 0, $cachedir) ) {
					$old = $cached;
					$old = json_decode($cached);
				}
			}
		}
		
		// build query
		$this->query->clear();
		$this->query->set($method);
		
		if ( count($vars) ) {
			$this->query->wheres($vars);
		}

		// try fetching results		
		try {					
			$return = $this->request->setQuery( $this->query )->getResults($key, false, $returnfalse, $log);	
			
			if ( $return ) {
				$found	= true;	
			}
			else {
				if ( $returnfalse && $return === false ) {
					return false;
				}
				else {
					$return = $old;
				}
			}
		}
		catch ( BookzoRequestException $e ) {
			$return = $old;
		}
		
		// do we need to cache
		if ( $cache && $found ) 
		{
			$this->app->cache->cache( $cachename, json_encode($return), $cachedir );
		}
		
		// return
		return $return;
	}
	
	protected function getResult( $method, $vars = array(), $key = null, $default = false, $cache = true, $cachetime = null, $cachedir = 'queries', $log = true )
	{
		$return = $this->getResults($method, $vars, $key, $default, $cache, $cachetime, $cachedir, $log);
		
		if ( !count($return) ) {
			return $default;
		}
		
		return is_array($return) ? reset($return) : null;
	}
	
	/**
	 * Method to turn bookzo response data into desired class
	 * 
	 * @param 	results	array	an array of bookzo results found
	 * @param 	type	string	the class desired
	 * @param 	key		string	a alternative key for the return array
	 * @return 	array			a list of converted results
	 * @access	public
	 * @since 	1.0.0
	 */
	protected function getResponseData($results, $type = 'BookzoResponseData', $key = null)
	{
		$return = array();
		
		foreach ( $results as $id => $result ) 
		{
			try { 
				// does class exists
				if ( !class_exists($type) ) {
					throw new BzException('Class does not exists');
				}
				
				// turn result into desired class
				if ( $key ) {
					if ( isset($result->{$key})) {
						$return[$result->{$key}] = new $type($result);
					}
				}
				else {
					$return[$id] = new $type($result);
				}
			}
			catch ( Exception $e ) {
				continue;
			}
		}
		
		// return
		return $return;
	}
}