<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
class BzModelResults extends BModelList 
{	
	protected $query;
	
	/**
	 * Get the departure date by startdate and duration type
	 *
	 * @param	$start		string	the arrival date	 
	 * @param	$definition	int		the pricedefininition id
	 * @return	$departure	string	the departure date
	 * @access	public
	 * @since	1.0
	 */
	public function getDeparture($arrival, $definition)
	{
		$departure	 = false;
		
		// get the correct definition
		if ( $definition = $this->app->api->getPriceDefinitions( (int) $definition ) )
		{
			$departure	= $this->app->date->create( $arrival )->add( $definition->numberofnights )->format( 'Y-m-d' );
		}
		
		return $departure;
	}
	
	public function getItems()
	{	
		// try loading from cache
		if ( isset($this->cache['items']) ) {
			return $this->cache['items'];
		}
		
		// get search data
		$items				= array();
		$arrival 			= $this->getState('arrival');
		$departure 			= $this->getState('departure');
		$persons			= $this->getState('persons');
		$pets				= $this->getState('pets');
		$group				= $this->getState('group');
				
		// do we have the query in session cache
		## NOTE - Shouldn't we also check for same query filters etc
		$found	 	= $this->app->session->get('bookzo.search.results');	

		// no cache then make a new fetch
		if ( !is_array($found) || !count($found) || 1 == 1) 
		{
			$found	 	= $this->app->api->getFreeObjects(compact('arrival', 'departure', 'group', 'persons', 'pets'));				
			$this->app->session->set('bookzo.search.results', $found);
		}

		// get ids found
		$foundids	= array_keys($found);

		// if found find matching objects
		if ( count($foundids) ) 
		{	
			$objects	= $this->app->api->getObjects($foundids);

			// add prices to the objects
			foreach ( $objects as $id => $object ) {
				$object->addPrice($found[$id]->price);	
				
				$object->addPrice($found[$id]->price);			
				$object->set('searchresult', true);
			}

			$items = $objects;
		}
		
		## NOTE - should we group here
		// group results if requested
		if ( $this->app->config->get('bookzo.bookpergroup', 0) ) {
			
			$tmp 		 = array();
			$foundgroups = array();
			foreach ( $items as $k => $item ) {
				if ( !in_array($item->get('groupid'), $foundgroups) ) {
					$foundgroups[] = $item->get('groupid');
					$tmp[$k] = $item;
				}
			}
			
			$items = $tmp;
		}
		
		// last but not least filter the items
		$items = $this->app->bookzo->filters->addObject($items)->getFiltered();

		// add the filtered elements to the session
		$this->app->session->set('bookzo.results.list', array_keys($items));
		
		// sort the items as requested
		$this->order($items);

		// update cache
		$this->cache['items'] = $items;
		
		return $this->cache['items'];
	}
	
	public function buildSearchQuery()
	{
		if ( is_null($this->query) ) 
		{
			// get old query
			$old 			= $this->app->registry->create( $this->app->session->get('bookzo.search.query', array()) )->clean(true);
			
			// build new query
			$new 			= $this->app->registry->create();
			$new->set('type', 'search');
			$group  		= $new->set('group', 		$this->app->input->getInt('group', 		$old->get('group')));
			$arrival 		= $new->set('arrival', 		$this->app->input->getCmd('arrival', 	$old->get('arrival')));
			$departure		= $new->set('departure', 	$this->app->input->getCmd('departure', 	$old->get('departure')));
			$month			= $new->set('month', 		$this->app->input->getCmd('month', 		$old->get('month')));
			$persons		= $new->set('persons', 		$this->app->input->getInt('persons', 	$old->get('persons')));
			$pets			= $new->set('pets', 		$this->app->input->getInt('pets', 		$old->get('pets')));
			$duration		= $new->set('duration', 	$this->app->input->getCmd('duration', 	$old->get('duration')));
												
			$new->clean(true);
			
			// do we need to continue or are we ok
			if ( $new->compare($old, array('type','group','arrival','departure','persons','pets','aid')) ) {
				$return = $old;
			}
			else {
				// store data in the session and reset
				$this->app->session->set('bookzo.pagetype', 'search');
				$this->app->userstate->set('results.limitstart', 0);
				$this->app->session->clear('bookzo.search.results');
				$this->app->session->clear('bookzo.calendar.values');
				$this->app->session->clear('bookzo.results.filters');

				// reapply filters if they are set
				$filters = $this->app->input->get('filters', array(), 'array');
				if ( count($filters) ) {
					$this->app->session->set('bookzo.results.filters', $filters);
				}

				// determine arrival
				if ( !$arrival ) 
				{
					$arrival = $new->set('arrival', $this->app->api->getArrivals( compact('group', 'persons', 'pets'), true ));
				}

				// determine departure
				if ( !$departure || ($arrival != $old->get('arrival') || $duration != $old->get('duration') ) )
				{
					$departure = false;
					
					if ( $duration && ($departure = $this->getDeparture($arrival, $duration)) ) 
					{
						$new->set('departure', $departure);
					}
					
					if ( !$departure ) {
						$departure = $new->set('departure', $this->app->api->getDepartures( compact('arrival', 'group', 'persons', 'pets'), true ));
					}
				}

				// also store real query data in the session
				$this->app->session->set('bookzo.search.query', $new->toArray() );
				
				$return = $new;
			}
			
			// remove month, duration and type (we don't need it in url)
			$return->remove(array('month', 'duration', 'type'))->clean(true);
		
			$this->query = $return;			
		}
		
		return $this->query;
	}
	
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set.
	 *
	 * @return  void
	 *
	 * @note    Calling getState in this method will result in recursion.
	 * @since   12.2
	 */
	protected function populateState( $order = 'id', $dir = 'asc' )
	{
		// call parent populate for default settings
		parent::populateState( $this->app->config->get('template.results_order'), $this->app->config->get('template.results_order_dir') );
		
		$values = $this->buildSearchQuery();

		$this->state->set('arrival', 	$values->get('arrival'));
		$this->state->set('departure', 	$values->get('departure'));
		$this->state->set('persons', 	$values->get('persons', 1));
		$this->state->set('pets', 		(int) $values->get('pets', 0));
		$this->state->set('group', 		(int) $values->get('group', 0));
	}
}